#include "OctaveEngine.h"

OctaveEngine::OctaveEngine(int argc, char* argv[])
{
  int rc = octave_main(argc,argv,1);
  if (rc)
    printf ("Octave Engine started successfully\n");
}

OctaveEngine::~OctaveEngine()
{
  clean_up_and_exit(0);
  printf ("Octave Engine closed\n");
}

// Interaction functions
status_t OctaveEngine::createVariable (string varName, int height, int width)
{
  Matrix var(height,width);
  varMap[varName] = var;
  
  return STATUS_OK;
}

// Puts a NULL placeholder into the varMap
status_t OctaveEngine::declareVariable (string varName)
{
  Matrix var;
  varMap[varName] = var;
}


status_t OctaveEngine::printVariable (string varName)
{
  if (varMap.find(varName)!=varMap.end())
  {
    Matrix var = varMap[varName];
    
    // Put the new values into the mxArray
    int count = var.cols()*var.rows();
    cout << "VarName: " << varName << "     Size: (" << var.rows() << ", " << var.cols() << ")" << endl;
    for (int i = 0 ; i < count ; i++)
      cout << var(i) << " ";
    cout << endl;
  }
  return STATUS_OK;
}

// Deletes the mxArray data and removes from the varMap
status_t OctaveEngine::destroyVariable (string varName)
{
  if (varMap.find(varName)!=varMap.end())
  {
    varMap.erase(varName);
  }
  return STATUS_OK;
}

// Pushes whatever the current value is into Matlab.
status_t OctaveEngine::pushVariable (string varName)
{
  if (varMap.find(varName)!=varMap.end())
  {
    Matrix var = varMap[varName];

    symbol_table::symbol_record symrec (varName, var);

    // Insert your symbol_record into the symbol_table
    symbol_table::symbol_record& symrectable = symbol_table::insert(varName);
    symrectable=symrec;
    
  }
  else
  {
    printf("OctaveEngine::pushVariable - Unknown variable name\n");
    return STATUS_WARN;
  }
  
  return STATUS_OK;
}

// Pushes the supplied value into Matlab
status_t OctaveEngine::pushVariable (string varName, double* varDataPtr)
{
  if (varMap.find(varName)!=varMap.end())
  {
    Matrix var = varMap[varName];
    
    // Put the new values into the mxArray
    int count = var.cols()*var.rows();
    for (int i = 0 ; i < count ; i++)
      var(i) = varDataPtr[i];
    
    //// TODO: DO I NEED TO STORE BACK TO THE MAP?
    destroyVariable(varName);
    varMap[varName] = var;

    symbol_table::symbol_record symrec (varName, var);

    // Insert your symbol_record into the symbol_table
    symbol_table::symbol_record& symrectable = symbol_table::insert(varName);
    symrectable=symrec;
  }
  else
  {
    printf("OctaveEngine::pushVariable - Unknown variable name\n");
    return STATUS_WARN;
  }
  
  return STATUS_OK;
}

// Pulls the variable from Matlab
status_t OctaveEngine::pullVariable (string varName)
{
  octave_value var = symbol_table::varval(varName);
  
  // If there is a previous value in the map, free it and replace it
  destroyVariable(varName);
  varMap[varName] = var.matrix_value();

  return STATUS_OK;
}

// Pulls the variable from Matlab and returns the values
status_t OctaveEngine::pullVariable (string varName, double* varDataPtr)
{
  octave_value var = symbol_table::varval(varName);
  Matrix varm = var.matrix_value();

  // If there is a previous value in the map, free it and replace it
  destroyVariable(varName);
  varMap[varName] = varm;
  
  // TODO : Fill in varDataPtr with the data in retval
  int count = varm.rows()*varm.cols();
  for (int i = 0 ; i < count ; i++)
    varDataPtr[i] = varm(i);
  
  return STATUS_OK;
}

// Evaluates a command
status_t OctaveEngine::eval (string command, bool silent)
{
//   /* run the string in matlab */
  int status;
  eval_string(command,silent,status);
}

// Access functions
// Note that the user is expected to know the size of the buffere supplied and that things
// will fit.
// Simply gets the current value in the varMap
status_t OctaveEngine::getVariableData (string varName, void*& variablePtr)
{

}

// Simply sets the value in the varMap
status_t OctaveEngine::setVariableData (string varName, void*& variablePtr)
{

}
