/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2; coding: utf-8 -*- */
#ifndef __MATLAB_ENGINE_H__
#define __MATLAB_ENGINE_H__

// matlab engine include file
#include "octave/config.h"
#include "octave.h"

#include <octave/lo-mappers.h>
#include <octave/lo-utils.h>
#include <octave/mx-base.h>
#include <octave/str-vec.h>

#include <octave/defun-dld.h>
#include <octave/error.h>
#include <octave/gripes.h>
#include <octave/oct-obj.h>
#include <octave/ops.h>
#include <octave/ov-base.h>
#include <octave/ov-typeinfo.h>
#include <octave/ov.h>
#include <octave/ov-scalar.h>
#include <octave/pager.h>
#include <octave/pr-output.h>
#include <octave/symtab.h>
#include <octave/variables.h>
#include <octave/toplev.h>
#include <octave/parse.h>

// Standard C++ includes
#include <iostream>
#include <string>
#include <vector>
#include <map>
using namespace std;

typedef int status_t;

#define STATUS_OK    0
#define STATUS_ERR  -1
#define STATUS_WARN  1

class OctaveEngine
{
 private:

	map <string,Matrix> varMap;

 public:
  
  OctaveEngine(int argc, char* argv[]);
  ~OctaveEngine();

	// Interaction functions
	status_t createVariable (string varName, 
													 int height, int width);

	// Puts a NULL placeholder into the varMap
	status_t declareVariable (string varName);
	

	status_t printVariable (string varName);
	
	// Deletes the mxArray data and removes from the varMap
	status_t destroyVariable (string varName);
	
	// Pushes whatever the current value is into Matlab.
	status_t pushVariable (string varName);
	
	// Pushes the supplied value into Matlab
	status_t pushVariable (string varName, double* varDataPtr);
	
	// Pulls the variable from Matlab
	status_t pullVariable (string varName);
	
	// Pulls the variable from Matlab and returns the values
	status_t pullVariable (string varName, double* varDataPtr);
	
	// Evaluates a command
	status_t eval (string command, bool silent = true);
	
	
	// Access functions
	// Note that the user is expected to know the size of the buffere supplied and that things
	// will fit.
	// Simply gets the current value in the varMap
	status_t getVariableData (string varName, void*& variablePtr);

	// Simply sets the value in the varMap
	status_t setVariableData (string varName, void*& variablePtr);
};



#endif // __MATLAB_ENGINE_H__
